#ifndef LIST_H
#define LIST_H

#include <iostream>

// Egyirányú, fejelemes láncolt lista.
class List
{
	// Publikus rész (implementáció).
	public:
		// Kivételek.
		enum ListException
		{
			// Azt az esetet jelöli, amikor a lista üres, de az adott művelet így nem értelmezhető.
			Empty
		};

		// Konstruktor: létrehozza a láncolt lista fejelemét és beállítja a mutatókat.
		List();
		// Copy konstruktor: lemásolja az argumentumban megadott objektumot.
		List(const List &l);
		// Destruktor, a HEAP-en lefoglalt terület felszabadítása.
		virtual ~List();
		// Értékadás operátor: lemásolja az argumentumban megadott objektumot, ha nem saját magát adtuk értékül.
		List &operator=(const List &ol);
		
		// Elem hozzáfűzése a láncolt lista végére.
		void append(const double &v);
		// Elem befűzése a láncolt lista elejére.
		void prepend(const double &v);
		// A láncolt listában tárolt maximális érték megkeresése. Ha a lista üres, akkor kivétel kiváltása.
		double getMax() const throw(ListException);
		
		// Kiíró operátor baráttá tétele, a láncolt lista elemeinek felsorolásához.
		friend std::ostream &operator<<(std::ostream &out, const List &l);
		
	// Privát rész (reprezentáció).
	private:
		// A láncolt lista elemeinek reprezentációja.
		struct Node
		{
			// Tárolt érték.
			double value;
			// Következő elem mutatója.
			Node* next;
			
			// Konstruktor: létrehoz egy új láncolt lista elemet az alapértelmezett értékkel.
			Node(Node *n = NULL) : next(n) {}
			// Konstruktor: létrehoz egy új láncolt lista elemet a megadott értékkel.
			Node(const double &v, Node *n = NULL) : value(v), next(n) {}
		};
		
		// Fejmutató és az utolsó elem mutatója.
		Node *header, *last;
};

// Kiíró operátor deklarálása, a láncolt lista elemeinek felsorolása.
std::ostream &operator<<(std::ostream &out, const List &l);

#endif
